From be16ab41473e71b3bc7e338621d5135014a53f26 Mon Sep 17 00:00:00 2001
From: Michael Weber <michaelw@foldr.org>
Date: Mon, 28 Jul 2008 14:55:22 +0200
Subject: [PATCH] More intercession options for user code.

swank-backend.lisp (call-with-request-environment): new interface function
(call-with-slime-restarts): new interface function

swank.lisp: Use them
---
 swank-backend.lisp |    9 +++++++++
 swank-sbcl.lisp    |    2 +-
 swank.lisp         |    4 +++-
 3 files changed, 13 insertions(+), 2 deletions(-)

diff --git a/swank-backend.lisp b/swank-backend.lisp
index e833b9a..a9d86aa 100644
--- a/swank-backend.lisp
+++ b/swank-backend.lisp
@@ -32,6 +32,7 @@
            #:unbound-slot-filler
            #:declaration-arglist
            #:type-specifier-arglist
+           #:call-with-request-environment
            ;; inspector related symbols
            #:emacs-inspect
            #:label-value-line
@@ -601,6 +602,14 @@ HOOK should be called for both BREAK and INVOKE-DEBUGGER."
   (let ((*debugger-hook* hook))
     (funcall fun)))
 
+(definterface call-with-slime-restarts (function)
+  "Call FUNCTION with SLIME specific restarts active."
+  (funcall function))
+
+(definterface call-with-request-environment (function)
+  "Calls FUNCTION within a per-request environment."
+  (funcall function))
+
 (define-condition sldb-condition (condition)
   ((original-condition
     :initarg :original-condition
diff --git a/swank-sbcl.lisp b/swank-sbcl.lisp
index 18ba695..a12c2a5 100644
--- a/swank-sbcl.lisp
+++ b/swank-sbcl.lisp
@@ -793,7 +793,7 @@ Return a list of the form (NAME LOCATION)."
                       (signal (make-condition
                                'sldb-condition
                                :original-condition condition)))))
-      (funcall debugger-loop-fn))))
+      (call-with-slime-restarts debugger-loop-fn))))
 
 #+#.(swank-backend::sbcl-with-new-stepper-p)
 (progn
diff --git a/swank.lisp b/swank.lisp
index 4845997..1c62427 100644
--- a/swank.lisp
+++ b/swank.lisp
@@ -786,7 +786,9 @@ of the toplevel restart."
     (with-connection (connection)
       (with-simple-restart (abort "Return to SLIME's top level.")
         (let ((*sldb-quit-restart* (find-restart 'abort)))
-          (read-from-emacs))))))
+          (call-with-request-environment
+           (lambda ()
+             (read-from-emacs))))))))
 
 (defun current-socket-io ()
   (connection.socket-io *emacs-connection*))
-- 
1.6.0.rc1.2.ge0842b

